#define WIN32_LEAN_AND_MEAN
#define NOSERVICE
#define NOMCX

#define _WIN32_WINNT 0x0500

#define IDT_MYTIMER 100
#define IDC_MSGLABEL 20001
#define IDLESTARTTIME 15

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>

const wchar_t g_className[] = L"rInactiveLogoff";

HWND hWnd, hMsgLabel;
LASTINPUTINFO lif;
int seconds, idletime, timeleft;
bool idleMsgShowing = false;
wchar_t msgBuffer[128];

LRESULT CALLBACK MainWndProc(HWND, UINT, WPARAM, LPARAM);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	// Parse Command Line
	seconds = atoi(lpCmdLine);
	seconds += IDLESTARTTIME; // Add the initial 15 seconds here

	// Random Variables
	WNDCLASSEX wcx;
	MSG msg;

	// Get desktop size
	RECT waRect;
	SystemParametersInfo(SPI_GETWORKAREA, 0, &waRect, 0);

	int sWidth = waRect.right - waRect.left;
	int sHeight = waRect.bottom - waRect.top;

	// Window Class
	wcx.cbSize				= sizeof(WNDCLASSEX);
	wcx.style				= 0;
	wcx.lpfnWndProc			= MainWndProc;
	wcx.cbClsExtra			= 0;
	wcx.cbWndExtra			= 0;
	wcx.hInstance			= hInstance;
	wcx.hIcon				= LoadIcon(NULL, IDI_APPLICATION);
	wcx.hCursor				= LoadCursor(NULL, IDC_ARROW);
	wcx.hbrBackground		= (HBRUSH)(COLOR_BTNFACE+1);
	wcx.lpszMenuName		= NULL;
	wcx.lpszClassName		= g_className;
	wcx.hIconSm				= LoadIcon(NULL, IDI_APPLICATION);

	if(!RegisterClassEx(&wcx))
	{
		MessageBox(NULL, L"Window Registration Failed.", L"Error", MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}

	hWnd = CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST, g_className, L"Inactivity Notice", WS_CAPTION, sWidth - 352, sHeight - 112, 320, 80, NULL, NULL, hInstance, NULL);
	if(hWnd == NULL)
	{
		MessageBox(NULL, L"Window Creation Failed.", L"Error", MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}

	hMsgLabel = CreateWindow(L"STATIC", L"", WS_CHILD | WS_VISIBLE | SS_CENTER, 0, 20, 320, 40, hWnd, (HMENU) IDC_MSGLABEL, hInstance, NULL);
	SendMessage (hMsgLabel, WM_SETFONT, (WPARAM)GetStockObject(DEFAULT_GUI_FONT), TRUE);

	SetTimer(hWnd, IDT_MYTIMER, 1000, (TIMERPROC) NULL);
	lif.cbSize = sizeof(LASTINPUTINFO);

	UpdateWindow(hWnd);

	while(GetMessage(&msg, NULL, 0, 0) > 0)
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return 0;
}

LRESULT CALLBACK MainWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
		case WM_TIMER: // We know which timer it is so assume IDT_MYTIMER
			GetLastInputInfo(&lif);
			idletime = (GetTickCount() - lif.dwTime) / 1000;
			if(idletime > IDLESTARTTIME) {
				if(!idleMsgShowing) {
					ShowWindow(hWnd, SW_SHOW);
					idleMsgShowing = true;
				}
				if(idletime > seconds) {
					// Logoff
					ExitWindowsEx(EWX_LOGOFF, SHTDN_REASON_MAJOR_OTHER | SHTDN_REASON_MINOR_OTHER | SHTDN_REASON_FLAG_PLANNED);
				} else {
					// Update message
					timeleft = seconds - idletime;
					swprintf(msgBuffer, 128, L"Automatic log-off in %d seconds due to idling", timeleft);
					SetWindowText(hMsgLabel, (LPCWSTR)msgBuffer);
				}
			} else {
				// Hide idle window
				if(idleMsgShowing) {
					ShowWindow(hWnd, SW_HIDE);
					idleMsgShowing = false;
				}
			}
			break;
		case WM_CLOSE:
			KillTimer(hwnd, IDT_MYTIMER);
			DestroyWindow(hwnd);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hwnd, msg, wParam, lParam);
	}
	return 0;
}